import java.util.*;

public class Main {
	public static void main(String[] args) {
		// Get the data from the file
		List<String> data = ReadFile.open("test.txt");
		
		// Gets a map with the total occurrences for each domain
		Map<String, Integer> domains = getDomainTotal(data);

		System.out.println(domains);

		// Order the map by descending value of the value
		List<Map.Entry<String, Integer>> sortedDomains = sortDomainMapDesc(domains);
		
		System.out.println(sortedDomains);
	}

	/**
	 * Totals up the amount of times an email domain occured in the list. 
	 * @param emails A list of emails
	 * @return A map of email domains and the total amount of times they occured in the <code>emails</code> parameter
	 */
	private static Map<String, Integer> getDomainTotal(List<String> emails) {
		// Create a map of all email address domains and counting them
		Map<String, Integer> domains = new HashMap<>();
		for (String email : emails) {
			if (!EMail.validate(email)) {
				throw new IllegalArgumentException("There was an invalid email in the file!");
			}

			String domain = EMail.getDomain(email);

			// Add email domain to the list
			if (!domains.containsKey(domain)) {
				// Creates new entry in the map if the domain didn't exist before
				domains.put(domain, 1);
			} else {
				// Updates the map with the new total for the domain
				// Gets the old total and adds 1
				Integer newTotal = domains.get(domain) + 1;

				// Update domain value with new total
				domains.replace(domain, newTotal);
			}
		}
		
		return domains;
	}

	/**
	 * Sorts the domains in descending order, from most occurrences to the least occurrences of domains.
	 * @param domains An unsorted map of domains with the amount that they occur
	 * @return Sorted list of Map Entries, in descending order of occurrences
	 */
	private static List<Map.Entry<String, Integer>> sortDomainMapDesc(Map<String, Integer> domains) {
		// Create a list from the map entries
		List<Map.Entry<String, Integer>> sortedDomains = new ArrayList<>(domains.entrySet());
		
		// Sort the list in descending order
		sortedDomains.sort(Map.Entry.comparingByValue(Comparator.reverseOrder()));
		
		return sortedDomains;
	}
}